/*******************************************************************************
********************************************************************************

   COPYRIGHT  2005, Nathaniel Group, Inc.

   Contains confidential and proprietary information which may not be copied,
   disclosed or used by others except as expressly authorized in writing
   by Nathaniel Group.

   tk_crc.c       Implementation for multiple CRC calculations

   About:         This file contains 8-bit and 16-bit CRC calculations
                  Review defines to configure table-driven or run-time
                  CRC calculation

   Target CPU:    N/A

   Written for:   Nathaniel Group
                  101 Panton Road, Suite 1
                  Vergennes, VT 05491-0173
                  http://www.nathaniel.com

   Contacts:      Joel Melnick
                  Phone:  (802) 877-2182
                  FAX:    (802) 877-3817
                  jmelnick@nathaniel.com

   Written by:    James Piette
                  Microprocessor Designs, Inc.
                  65 Longmeadow Drive
                  PO Box 160
                  Shelburne, VT 05482
                  http://www.updesigns.com

   $Revision: 2 $

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
   $Log: /Advanced Illumination/Pulsar320/pc_src/MPDH8USB_TCP/tk_crc.c $
 * 
 * 2     8/15/08 12:34p Jpiet
 * 
 * 1     8/15/08 8:45a Jpiet
 * Initial check-in
 * 
 * 1     5/12/06 2:38p Jpiet
 * 
 * 9     5/09/06 2:36p Jpiet
 * 
 * 8     5/09/06 2:18p Jpiet
 * Problems with conditionals.
 * 
 * 7     5/09/06 2:05p Jpiet
 * Fixed conditionals to make files compatible with all builds (console,
 * camera and PC)
 *
 * 6     5/09/06 12:51p Jpiet
 * Put CRC16 function in flash for bootloader build
 *
 * 5     5/08/06 9:39a Jpiet
 * Fixed conditionals for both PC and target builds.
 *
 * 4     5/08/06 9:26a Jpiet
 * Fixed problems with CRC16 function syntax
 *
 * 3     5/05/06 4:17p Jpiet
 * Put crc in flash for bootloader builds.
 *
 * 2     1/09/06 9:53a Jpiet
 * Most function implemented.  Working on EEPROM access interface.  Making
 * bootloader modification, so all shared files checked in from APP
 * changes.
 *
 * 1     12/30/05 5:31a Jpiet
********************************************************************************
*******************************************************************************/
#include "compiler.h"
#include "tk_crc.h"
//#include "devmap.h"
#ifndef WIN32
#include "macros.h"
#endif
/*******************************************************************************
 *******************************************************************************
                     Definitions
********************************************************************************
*******************************************************************************/
#define TKCRC_CRC8_POLY       0x07  //X^8 + X^2 + X^1 + 1 (CRC8 ATM)
#define TKCRC_CRC16_POLY      0x07  //X^8 + X^2 + X^1 + 1 (CRC8 ATM)

/*******************************************************************************
 *******************************************************************************
                           Globals
********************************************************************************
*******************************************************************************/
#if TKCRC_USE_TABLES
const UINT8_T crc8table[] = {
   0x00,0x07,0x0E,0x09,0x1C,0x1B,0x12,0x15,0x38,0x3F,0x36,0x31,0x24,0x23,0x2A,0x2D,
   0x70,0x77,0x7E,0x79,0x6C,0x6B,0x62,0x65,0x48,0x4F,0x46,0x41,0x54,0x53,0x5A,0x5D,
   0xE0,0xE7,0xEE,0xE9,0xFC,0xFB,0xF2,0xF5,0xD8,0xDF,0xD6,0xD1,0xC4,0xC3,0xCA,0xCD,
   0x90,0x97,0x9E,0x99,0x8C,0x8B,0x82,0x85,0xA8,0xAF,0xA6,0xA1,0xB4,0xB3,0xBA,0xBD,
   0xC7,0xC0,0xC9,0xCE,0xDB,0xDC,0xD5,0xD2,0xFF,0xF8,0xF1,0xF6,0xE3,0xE4,0xED,0xEA,
   0xB7,0xB0,0xB9,0xBE,0xAB,0xAC,0xA5,0xA2,0x8F,0x88,0x81,0x86,0x93,0x94,0x9D,0x9A,
   0x27,0x20,0x29,0x2E,0x3B,0x3C,0x35,0x32,0x1F,0x18,0x11,0x16,0x03,0x04,0x0D,0x0A,
   0x57,0x50,0x59,0x5E,0x4B,0x4C,0x45,0x42,0x6F,0x68,0x61,0x66,0x73,0x74,0x7D,0x7A,
   0x89,0x8E,0x87,0x80,0x95,0x92,0x9B,0x9C,0xB1,0xB6,0xBF,0xB8,0xAD,0xAA,0xA3,0xA4,
   0xF9,0xFE,0xF7,0xF0,0xE5,0xE2,0xEB,0xEC,0xC1,0xC6,0xCF,0xC8,0xDD,0xDA,0xD3,0xD4,
   0x69,0x6E,0x67,0x60,0x75,0x72,0x7B,0x7C,0x51,0x56,0x5F,0x58,0x4D,0x4A,0x43,0x44,
   0x19,0x1E,0x17,0x10,0x05,0x02,0x0B,0x0C,0x21,0x26,0x2F,0x28,0x3D,0x3A,0x33,0x34,
   0x4E,0x49,0x40,0x47,0x52,0x55,0x5C,0x5B,0x76,0x71,0x78,0x7F,0x6A,0x6D,0x64,0x63,
   0x3E,0x39,0x30,0x37,0x22,0x25,0x2C,0x2B,0x06,0x01,0x08,0x0F,0x1A,0x1D,0x14,0x13,
   0xAE,0xA9,0xA0,0xA7,0xB2,0xB5,0xBC,0xBB,0x96,0x91,0x98,0x9F,0x8A,0x8D,0x84,0x83,
   0xDE,0xD9,0xD0,0xD7,0xC2,0xC5,0xCC,0xCB,0xE6,0xE1,0xE8,0xEF,0xFA,0xFD,0xF4,0xF3
};
#endif
#if TKCRC_INCREMENTAL_CRC
   #if TKCRC_CRC8
      STATIC UINT8_T  tkcrc_running_crc8;
   #endif
   #if TKCRC_CRC16
      STATIC UINT16_T tkcrc_running_crc16;
      STATIC UINT16_T tkcrc_crc16_coeff;
   #endif
#endif


#if TKCRC_CRC8
/*******************************************************************************
   tkcmdpr_calc_crc8

   About:      Calculates an 8-bit CRC using the X^8+X^2+X^1+1 polynomial (0x07)

   Inputs:     UINT8_T  *bufptr     Beginning of buffer for which checksum
                                    is calculated
               UINT8_T  num         Number of bytes to compute CRC on

   Outputs:

   Returns:    0 if successful, or negative value error code

*******************************************************************************/
UINT8_T tkcrc_calc_crc8(UINT8_T *bufptr, UINT16_T num_bytes)
{
   /* Initialize CRC value */
   tkcrc_running_crc8 = 0;

   /* Add each byte of the response to the CRC */
   while(num_bytes--)
   {
   #if TKCRC_USE_TABLES
      /* Table driven CRC */
      tkcrc_running_crc8 = crc8table[tkcrc_running_crc8 ^ *bufptr++];
   #else
      /* Code driven CRC */
      data = *bufptr++;
      for(i = 0; i < 8; i++)
      {
         if((tkcrc_running_crc8 ^ data) & 0x80)
         {
            tkcrc_running_crc8 = (tkcrc_running_crc8 << 1) ^ TKCRC_CRC8_POLY;
         }
         else
         {
            tkcrc_running_crc8 <<= 1;
         }
         data <<= 1;
      }
   #endif	
   }
   return(tkcrc_running_crc8);
}


   #if TKCRC_INCREMENTAL_CRC
/****************************************************************************
    tkcrc_reset_crc8

    About:      Resets the running 8-bit CRC

    Inputs:     None

    Outputs:    None

    Returns:    void

****************************************************************************/
void tkcrc_reset_crc8()
{
   /* Initialize CRC to zero */
   tkcrc_running_crc8 = 0;
}

/****************************************************************************
    tkcrc_add_val8

    About:      Add a new 8-bit value to the currently running CRC

    Inputs:     UINT8_T   val          Value to be added

    Outputs:    None

    Returns:    void

****************************************************************************/
void tkcrc_add_crc8(UINT8_T val)
{

#if TKCRC_USE_TABLES
   /* Table driven CRC */
   tkcrc_running_crc8 = crc8table[tkcrc_running_crc8 ^ val];
#else
   UINT8_T i;
   for(i = 0; i < 8; i++)
   {
      if((tkcrc_running_crc8 ^ val) & 0x80)
      {
         tkcrc_running_crc8 = (tkcrc_running_crc8 << 1) ^ TKCRC_CRC8_POLY;
      }
      else
      {
         tkcrc_running_crc8 <<= 1;
      }
      val <<= 1;
   }
#endif
}

/****************************************************************************
    tkcrc_get_crc8

    About:      Gets the current running 8-bit CRC total

    Inputs:     None

    Outputs:    UINT8_T   *crc8      Location to store current running CRC

    Returns:    UINT8_T   current value of running 8-bit crc

****************************************************************************/
UINT8_T tkcrc_get_crc8()
{
   return(tkcrc_running_crc8);
}

   #endif //TKCRC_INCREMENTAL_CRC
#endif //TKCRC_CRC8


#if TKCRC_CRC16
/****************************************************************************
    tkcrc_add_val16

    About:      Add a new 16-bit value to the currently running CRC

    Inputs:     UINT8_T   val          Value to be added

    Outputs:    None
    
    Returns:    ERRCODE_T  ecode       E_SUCCESS if successful
                                       Error code otherwise

****************************************************************************/
ERRCODE_T tkcrc_add_val16(UINT8_T val)
{
   ERRCODE_T ecode = E_SUCCESS;
   UINT8_T i;
   
   for (i = 0; i < 8; i++)
   {
      if ((tkcrc_running_crc16 ^ val) & 1)
      {
         tkcrc_running_crc16 = (tkcrc_running_crc16 >> 1) ^ tkcrc_crc16_coeff;
      }
      else
      {
         tkcrc_running_crc16 >>= 1;
      }
      val >>= 1;
   }
   
   return ecode;
}

/****************************************************************************
    tkcrc_get_crc16

    About:      Gets the current running 16-bit CRC total

    Inputs:     None

    Outputs:    UINT16_T   *crc16      Location to store current running CRC
    
    Returns:    ERRCODE_T  ecode       E_SUCCESS if successful
                                       Error code otherwise

****************************************************************************/
ERRCODE_T tkcrc_get_crc16(UINT16_T *crc16)
{
   ERRCODE_T ecode = E_SUCCESS;
   
   *crc16 = tkcrc_running_crc16;
   
   return ecode;
}


/****************************************************************************
    tkcrc_calc16

    About:      Computes the 16-bit CRC of an entire block

    Inputs:     UINT8_T    *val_ptr    Location of data for calculation
                UINT32_T   num_bytes   Number of bytes in data block

    Outputs:    UINT16_T   *crc16      Location to store final CRC16 value
    
    Returns:    ERRCODE_T  ecode       E_SUCCESS if successful
                                       Error code otherwise

****************************************************************************/
ERRCODE_T tkcrc_calc16(UINT8_T *val_ptr, UINT32_T num_bytes, UINT16_T *crc16)
{
   ERRCODE_T ecode = E_SUCCESS;

   /* Initialize CRC values */
   tkcrc_running_crc16 = 0;
   tkcrc_crc16_coeff = TKCRC_CRC16_DEFAULT_COEFF;

   /* Add all the bytes to the sum */
   while (num_bytes--)
   {
      if (ecode = TKCRC_ADD_VAL16_M(*val_ptr++))
      {
         break;
      }
   }
   
   /* Return the final value */ 
   if (ecode)
   {
      *crc16 = 0;
   }
   else if (ecode = TKCRC_GET_CRC16_M(crc16))
   {
   }
   
   return ecode;
}
#endif //TKCRC_CRC16
