/*******************************************************************************
********************************************************************************

   COPYRIGHT  2004, Advanced Illumination, Inc. 

   Contains confidential and proprietary information which may not be copied,
   disclosed or used by others except as expressly authorized in writing
   by Advanced Illumination.

   tk_time.cpp    Source code for the time management toolkit functions

   About:         This module contains the implementation of the time management
                  toolkit functions of the S1000 Strobe Controller in 
                  the AICONS01 project.

   Target:        Win32 Compatible

   Written for:   Advanced Illumination, Inc.
                  24 Peavine Drive
                  Rochester, VT 05767
                  http://advancedillumination.com

   Contacts:      John Thrailkill
                  Phone:  (802) 767-3830
                  FAX:    (802) 767-3831
                  jthrailkill@advill.com

   Written by:    Rob Macklin
                  Microprocessor Designs, Inc.
                  65 Longmeadow Drive
                  PO Box 160
                  Shelburne, VT 05482
                  http://www.updesigns.com

   $Revision: 2 $

--------------------------------------------------------------------------------
   $Log: /Advanced Illumination/S1000/pc_src/Common/tk_time.cpp $
 * 
 * 2     8/24/04 8:44a Rmack
 * Updated header information.
 * 
 * 1     3/30/04 3:27p Rmack
 * 
********************************************************************************
*******************************************************************************/
#include "stdafx.h"
#include <mmsystem.h>
#include "compiler.h"
#include "pc_error.h"
#include "tk_time.h"


STATIC LARGE_INTEGER tktime_hp_freq = {0};


/****************************************************************************
   tktime_init

   About:      Initializes the Time Toolkit functions

   Inputs:     None

   Outputs:    None

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
ERRCODE_T tktime_init(void)
{
   ERRCODE_T ecode = NO_ERR;

   /* Determine resolution of high performance counter */
   if (!QueryPerformanceFrequency(&tktime_hp_freq))
   {
      ecode = TKTIME_NO_HIRES_ERR;
   }

   return ecode;
}


/****************************************************************************
   tktime_delay_ms

   About:      Delays execution for the specified number of milliseconds.
               A message loop is implemented to yield processor time back
               to Windows while delaying.

   Inputs:     UINT32_T    ms          Number of milliseconds to delay

   Outputs:    None

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
ERRCODE_T tktime_delay_ms(UINT32_T ms)
{
   ERRCODE_T ecode = NO_ERR;
   UINT32_T start_time = TKTIME_GETTIME_M();
   MSG msg;

   do
   {
      /* Process message queue while waiting */
      while ( PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
      {
         if (!GetMessage(&msg, NULL, NULL, NULL))
         {
            ms = 0; 
            ecode = TKTIME_ABORT_ERR;
            PostQuitMessage(0); 
            break; 
         }
         else
         {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
         }
      }
    }
   /* Wait until the desired time has elapsed */
    while (TKTIME_ELAPSED_TIME_MS_M(start_time) <= ms);

    return ecode;
}

/****************************************************************************
   tktime_delay_us

   About:      Delays execution for the specified number of microseconds.
               No message loop is implemented, this function does not
               yield to Windows while delaying.

   Inputs:     UINT32_T    us          Number of microseconds to delay

   Outputs:    None

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
ERRCODE_T tktime_delay_us(UINT32_T usecs)
{
   ERRCODE_T ecode = NO_ERR;
   LARGE_INTEGER endcount;
   LARGE_INTEGER nowcount;

   /* Get current hi resolution count value */
   if (!QueryPerformanceCounter(&endcount))
   {
      TKTIME_DELAY_MS_M(TKTIME_ONE_SEC);
      ecode = TKTIME_NO_HIRES_ERR;
   }
   /* Wait until the desired number of microseconds has elapsed */
   else
   {
      /* Compute the end count value */
      endcount.QuadPart += (usecs * tktime_hp_freq.QuadPart) / 1000000;

      /* Poll the high performance counter until the end count is exceeded */
      do
      {
         QueryPerformanceCounter(&nowcount);
      }
      while (nowcount.QuadPart <= endcount.QuadPart);
   }

   return ecode;
}
